/***************************************************************************
    file	         : sql.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qfile.h>
#include	<qstring.h>
#include	<qmap.h>

#include	"kb_dblink.h"
#include	"kb_database.h"
#include	"kb_databuffer.h"

#include	"tester.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif


static	unsigned short	binary1[] =
{
0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007, 
0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017
}	;

class	KBDBTest : public CppUnit::TestCase
{
	QDomElement	m_elem		;

	KBServer	*m_server	;
	KBDBLink	*m_dbLink	;
	KBServerInfo	*m_serverInfo	;

public	:

	KBDBTest	() ;
	KBDBTest	(std::string name, const QDomElement &) ;

	virtual	void		setUp			() ;
	virtual	void		tearDown		() ;

	void			testMapExpression	() ;
	void			testSubPlaceList	() ;
	void			testEscapeText		() ;

	static CppUnit::Test	*suite			() ;
}	;


KBDBTest::KBDBTest ()
{
	m_server = 0 ;
}

KBDBTest::KBDBTest
	(	std::string		name,
		const QDomElement	&elem
	)
	:
	CppUnit::TestCase 	(name),
	m_elem			(elem)
{
	m_server = 0 ;
}

void	KBDBTest::setUp ()
{
	extern	KBServer *createTestDriver (const QMap<QString,QString> &) ;
	QMap<QString,QString> options ;
	options["mapExpressions"] = m_elem.attribute("map", "0") ;
	m_server = createTestDriver (options) ;
}

void	KBDBTest::tearDown ()
{
	if (m_server != 0) delete m_server ;
}

void	KBDBTest::testMapExpression ()
{
	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(m_elem.attribute("name"  )),
		_STDSTR(m_elem.attribute("result")),
		_STDSTR(m_server->mapExpression(m_elem.attribute("test")))
	)	;
}

void	KBDBTest::testSubPlaceList ()
{
	uint		nvals	= 0 ;
	KBValue		values[32]  ;
	KBDataBuffer	buffer	    ;
	KBError		error	    ;

	DOMITER_BEGIN(m_elem,"value",elem)
	{
		CPPUNIT_ASSERT(nvals < 32) ;

		QString	type	= elem.attribute ("type") ;

		if	(type == "fixed")
			values[nvals] = KBValue (elem.text().toInt   ()) ;
		else if	(type == "float")
			values[nvals] = KBValue (elem.text().toDouble()) ;
		else if (type == "datetime")
			values[nvals] = KBValue (elem.text(), &_kbDateTime) ;
		else if (type == "binary")
			values[nvals] = KBValue (elem.text(), &_kbBinary) ;
		else if (type == "binary1")
			values[nvals] = KBValue ((cchar *)binary1, sizeof(binary1), &_kbBinary) ;
		else	
			values[nvals] = KBValue (elem.text()) ;

		nvals	  += 1 ;
	}
	DOMITER_END

	bool	rc	= m_server->subPlaceList
			  (	m_elem.attribute("sql"),
				nvals,
				values,
				buffer,
				0,
				error
			  )	;

	if (m_elem.attribute("rc").toInt() == 0)
	{
		CPPUNIT_ASSERT (!rc) ;
		return	;
	}

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage()),
		rc
	)	;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(m_elem.attribute("name"  )),
		_STDSTR(m_elem.attribute("result")),
		_STDSTR(QString::fromUtf8(buffer.data(), buffer.length()))
	)	;
}

void	KBDBTest::testEscapeText ()
{
	QString		type	= m_elem.attribute ("type") ;
	KBValue		value	;
	KBDataBuffer	buffer	;

	if	(type == "fixed")
		value = KBValue (m_elem.text().toInt   ()) ;
	else if	(type == "float")
		value = KBValue (m_elem.text().toDouble()) ;
	else if (type == "datetime")
		value = KBValue (m_elem.text(), &_kbDateTime) ;
	else if (type == "binary")
		value = KBValue (m_elem.text(), &_kbBinary) ;
	else if (type == "binary1")
		value = KBValue ((cchar *)binary1, sizeof(binary1), &_kbBinary) ;
	else	
		value = KBValue (m_elem.text()) ;

	m_server->getQueryText (&value, buffer, 0) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Got [%1], expected [%2]").arg(buffer.data()).arg(m_elem.attribute("result").latin1())),
		qstrcmp(m_elem.attribute("result").latin1(), buffer.data()) == 0
	)	;
}


static	void	addMapExpressionTests
	(	CppUnit::TestSuite	*suite,
		const QString		&spec
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite(_STDSTR(spec)) ;
	suite->addTest (s) ;

	KBDomDocument	doc	;
	if (!doc.loadFile (spec + ".xml", "test/common", getenv("TESTPATH")))
	{
		fprintf
		(	stderr,
			"db: %s: cannot open: %s\n",
			spec.latin1(),
			doc.lastError().getMessage().latin1()
		)	;
		exit	(1) ;
	}

	DOMITER_BEGIN(doc.documentElement(),"test",elem)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBDBTest>
			(	_STDSTR(elem.attribute("name")),
				&KBDBTest::testMapExpression,
				new KBDBTest (_STDSTR(elem.attribute("name")), elem)
		)	)	;
	}
	DOMITER_END
}

static	void	addSubPlaceListTests
	(	CppUnit::TestSuite	*suite,
		const QString		&spec
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite(_STDSTR(spec)) ;
	suite->addTest (s) ;

	KBDomDocument	doc	;
	if (!doc.loadFile (spec + ".xml", "test/common", getenv("TESTPATH")))
	{
		fprintf
		(	stderr,
			"db: %s: cannot open: %s.xml\n",
			spec.latin1(),
			doc.lastError().getMessage().latin1()
		)	;
		exit	(1) ;
	}

	DOMITER_BEGIN(doc.documentElement(),"test",elem)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBDBTest>
			(	_STDSTR(elem.attribute("name")),
				&KBDBTest::testSubPlaceList,
				new KBDBTest (_STDSTR(elem.attribute("name")), elem)
		)	)	;
	}
	DOMITER_END
}

static	void	addEscapeTextTests
	(	CppUnit::TestSuite	*suite,
		const QString		&spec
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite(_STDSTR(spec)) ;
	suite->addTest (s) ;

	KBDomDocument	doc	;
	if (!doc.loadFile (spec + ".xml", "test/common", getenv("TESTPATH")))
	{
		fprintf
		(	stderr,
			"db: %s: cannot open: %s.xml\n",
			spec.latin1(),
			doc.lastError().getMessage().latin1()
		)	;
		exit	(1) ;
	}

	DOMITER_BEGIN(doc.documentElement(),"test",elem)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBDBTest>
			(	_STDSTR(elem.attribute("name")),
				&KBDBTest::testEscapeText,
				new KBDBTest (_STDSTR(elem.attribute("name")), elem)
		)	)	;
	}
	DOMITER_END
}

CppUnit::Test
	*KBDBTest::suite ()
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("DB") ;

	if (!rekallTestSkip("DB"))
	{
		addMapExpressionTests(s, "mapexpr" ) ;
		addSubPlaceListTests (s, "subplace") ;
		addEscapeTextTests   (s, "escapetext") ;
	}

	return s ;
}

CppUnit::Test EXPORT_API *dbTestSuite
	(	const QMap<QString,QString> *
	)
{
	return	KBDBTest::suite () ;
}

