/***************************************************************************
    file	         : sql.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qfile.h>
#include	<qstring.h>
#include	<qmap.h>

#include	"kb_dblink.h"
#include	"kb_database.h"
#include	"kb_basequery.h"
#include	"kb_dom.h"

#include	"tester.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif

class	KBSQLTest : public CppUnit::TestCase
{
	QDomElement		m_elem		;
	KBServer		*m_server	;
	KBDBLink		*m_dbLink	;
	KBServerInfo		*m_serverInfo	;

	void			buildSelect	(const QDomElement &) ;
	void			buildInsert	(const QDomElement &) ;
	void			buildUpdate	(const QDomElement &) ;
	void			buildDelete	(const QDomElement &) ;
	void			buildSelectDOM	(const QDomElement &) ;
	void			buildInsertDOM	(const QDomElement &) ;
	void			buildUpdateDOM	(const QDomElement &) ;
	void			buildDeleteDOM	(const QDomElement &) ;
	void			evaluate	(const QDomElement &) ;
	void			evaluateDOM	(const QDomElement &) ;

public	:

	KBSQLTest	() ;
	KBSQLTest	(std::string name, const QDomElement &) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			testSQL		() ;
	void			testSQLDOM	() ;

	static CppUnit::Test	*suite		() ;
}	;


KBSQLTest::KBSQLTest ()
{
	m_server	= 0 ;
	m_dbLink	= 0 ;
	m_serverInfo	= 0 ;
}

KBSQLTest::KBSQLTest
	(	std::string		name,
		const QDomElement	&elem
	)
	:
	CppUnit::TestCase 	(name),
	m_elem			(elem)
{
	m_server	= 0 ;
	m_dbLink	= 0 ;
	m_serverInfo	= 0 ;
}

void	KBSQLTest::setUp ()
{
	extern	KBServer *createTestDriver (const QMap<QString,QString> &) ;
	QMap<QString,QString> options ;
	options["mapExpressions"] = m_elem.attribute("map", "0") ;

	m_server	= createTestDriver (options ) ;
	m_dbLink	= new KBDBLink	   ()	      ;
        m_serverInfo    = new KBServerInfo (0) 	      ;

	m_serverInfo->setServer ("test", m_server)    ;
	m_dbLink    ->connect   (m_serverInfo)	      ;
}

void	KBSQLTest::tearDown ()
{
	if (m_dbLink	 != 0) delete m_dbLink	   ;
	if (m_serverInfo != 0) delete m_serverInfo ;
//	if (m_server	 != 0) delete m_server     ;
}

void	KBSQLTest::buildInsert
	(	const QDomElement	&elem
	)
{
	KBBaseInsert	insert	;
	QString	 	id	= elem.attribute("id"    ) ;
	QString	 	expect	= elem.attribute("expect") ;

	DOMITER_ELEM_BEGIN(elem,child)
	{
		if (child.nodeName() == "table")
		{
			insert.setTable(child.attribute("name")) ;
			continue ;
		}
		if (child.nodeName() == "value")
		{
			QString type = child.attribute("type") ;

			if (type == "V")
			{
				insert.addValue (child.attribute("name")) ;
				continue ;
			}
			if (type == "S")
			{
				insert.addValue (child.attribute("name"), child.attribute("value")) ;
				continue ;
			}
			if (type == "D")
			{
				insert.addValue (child.attribute("name"), child.attribute("value").toInt()) ;
				continue ;
			}
			if (type == "F")
			{
				insert.addValue (child.attribute("name"), child.attribute("value").toDouble()) ;
				continue ;
			}
			if (type == "P")
			{
				insert.addValue (child.attribute("name"), (void *)0) ;
				continue ;
			}
			continue ;
		}
	}
	DOMITER_END

	QString	 result	= insert.getQueryText (m_dbLink) ;

	result	= result.simplifyWhiteSpace() ;
	expect	= expect.simplifyWhiteSpace() ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(id),
		_STDSTR(expect),
		_STDSTR(result)
	)	;
}

void	KBSQLTest::buildUpdate
	(	const QDomElement	&elem
	)
{
	KBBaseUpdate	update	;
	QString	 	id	= elem.attribute("id"    ) ;
	QString	 	expect	= elem.attribute("expect") ;

	DOMITER_ELEM_BEGIN(elem,child)
	{
		if (child.nodeName() == "table")
		{
			update.setTable(child.attribute("name")) ;
			continue ;
		}
		if (child.nodeName() == "value")
		{
			QString type = child.attribute("type") ;

			if (type == "V")
			{
				update.addValue (child.attribute("name")) ;
				continue ;
			}
			if (type == "S")
			{
				update.addValue (child.attribute("name"), child.attribute("value")) ;
				continue ;
			}
			if (type == "D")
			{
				update.addValue (child.attribute("name"), child.attribute("value").toInt()) ;
				continue ;
			}
			if (type == "F")
			{
				update.addValue (child.attribute("name"), child.attribute("value").toDouble()) ;
				continue ;
			}
			if (type == "P")
			{
				update.addValue (child.attribute("name"), (void *)0) ;
				continue ;
			}
			continue ;
		}
		if (child.nodeName() == "where")
		{
			QString	oper = child.attribute ("oper") ;
			QString type = child.attribute ("type") ;

			if (type == "V")
			{
				update.addWhere (child.attribute("name"), oper.latin1()) ;
				continue ;
			}
			if (type == "A")
			{
				update.addWhere (child.attribute("name"), KBBaseQueryExpr::m_asis) ;
				continue ;
			}
			if (type == "S")
			{
				update.addWhere (child.attribute("name"), child.attribute("value"), oper.latin1()) ;
				continue ;
			}
			if (type == "D")
			{
				update.addWhere (child.attribute("name"), child.attribute("value").toInt(), oper.latin1()) ;
				continue ;
			}
			if (type == "F")
			{
				update.addWhere (child.attribute("name"), child.attribute("value").toDouble(), oper.latin1()) ;
				continue ;
			}
			if (type == "P")
			{
				update.addWhere (child.attribute("name"), (void *)0, oper.latin1()) ;
				continue ;
			}
			continue ;
		}
	}
	DOMITER_END

	QString	 result	= update.getQueryText (m_dbLink) ;

	result	= result.simplifyWhiteSpace() ;
	expect	= expect.simplifyWhiteSpace() ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(id),
		_STDSTR(expect),
		_STDSTR(result)
	)	;
}

void	KBSQLTest::buildDelete
	(	const QDomElement	&elem
	)
{
	KBBaseDelete	delrec	;
	QString	 	id	= elem.attribute("id"    ) ;
	QString	 	expect	= elem.attribute("expect") ;

	DOMITER_ELEM_BEGIN(elem,child)
	{
		if (child.nodeName() == "table")
		{
			delrec.setTable(child.attribute("name")) ;
			continue ;
		}
		if (child.nodeName() == "where")
		{
			QString	oper = child.attribute ("oper") ;
			QString type = child.attribute ("type") ;

			if (type == "V")
			{
				delrec.addWhere (child.attribute("name"), oper.latin1()) ;
				continue ;
			}
			if (type == "A")
			{
				delrec.addWhere (child.attribute("name"), KBBaseQueryExpr::m_asis) ;
				continue ;
			}
			if (type == "S")
			{
				delrec.addWhere (child.attribute("name"), child.attribute("value"), oper.latin1()) ;
				continue ;
			}
			if (type == "D")
			{
				delrec.addWhere (child.attribute("name"), child.attribute("value").toInt(), oper.latin1()) ;
				continue ;
			}
			if (type == "F")
			{
				delrec.addWhere (child.attribute("name"), child.attribute("value").toDouble(), oper.latin1()) ;
				continue ;
			}
			if (type == "P")
			{
				delrec.addWhere (child.attribute("name"), (void *)0, oper.latin1()) ;
				continue ;
			}
			continue ;
		}
	}
	DOMITER_END

	QString	 result	= delrec.getQueryText (m_dbLink) ;

	result	= result.simplifyWhiteSpace() ;
	expect	= expect.simplifyWhiteSpace() ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(id),
		_STDSTR(expect),
		_STDSTR(result)
	)	;
}

void	KBSQLTest::buildSelect
	(	const QDomElement	&elem
	)
{
	KBBaseSelect	select	;
	QString		id	= elem.attribute("id"    ) ;
	QString		expect	= elem.attribute("expect") ;

	select.setDistinct  (elem.attribute("distinct").toInt()) ;
	select.setForUpdate (elem.attribute("update"  ).toInt()) ;


	DOMITER_ELEM_BEGIN(elem,child)
	{
		if (child.nodeName() == "table")
		{
			select.addTable(child.attribute("name"), child.attribute("alias")) ;
			continue ;
		}
		if (child.nodeName() == "fetch")
		{
			select.addFetch (child.attribute("expr"), child.attribute("alias")) ;
			continue ;
		}
		if (child.nodeName() == "where")
		{
			QString	oper = child.attribute ("oper") ;
			QString type = child.attribute ("type") ;

			if (type == "V")
			{
				select.addWhere (child.attribute("name"), oper.latin1()) ;
				continue ;
			}
			if (type == "A")
			{
				select.addWhere (child.attribute("name"), KBBaseQueryExpr::m_asis) ;
				continue ;
			}
			if (type == "S")
			{
				select.addWhere (child.attribute("name"), child.attribute("value"), oper.latin1()) ;
				continue ;
			}
			if (type == "D")
			{
				select.addWhere (child.attribute("name"), child.attribute("value").toInt(), oper.latin1()) ;
				continue ;
			}
			if (type == "F")
			{
				select.addWhere (child.attribute("name"), child.attribute("value").toDouble(), oper.latin1()) ;
				continue ;
			}
			if (type == "P")
			{
				select.addWhere (child.attribute("name"), (void *)0, oper.latin1()) ;
				continue ;
			}
			continue ;
		}
		if (child.nodeName() == "group")
		{
			select.addGroup(child.attribute("name")) ;
			continue ;
		}
		if (child.nodeName() == "having")
		{
			select.addHaving(child.attribute("name")) ;
			continue ;
		}
		if (child.nodeName() == "order")
		{
			select.addOrder(child.attribute("name")) ;
			continue ;
		}
	}
	DOMITER_END

	QString	 result	= select.getQueryText (m_dbLink) ;

	result	= result.simplifyWhiteSpace() ;
	expect	= expect.simplifyWhiteSpace() ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(id),
		_STDSTR(expect),
		_STDSTR(result)
	)	;
}

void	KBSQLTest::buildInsertDOM
	(	const QDomElement	&elem
	)
{
	KBBaseInsert	insert	(elem) ;
	QString	 	id	= elem.attribute("id"    ) ;
	QString	 	expect	= elem.attribute("expect") ;
	QString		result	= insert.getQueryText (m_dbLink) ;

	result	= result.simplifyWhiteSpace() ;
	expect	= expect.simplifyWhiteSpace() ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(id),
		_STDSTR(expect),
		_STDSTR(result)
	)	;
}

void	KBSQLTest::buildUpdateDOM
	(	const QDomElement	&elem
	)
{
	KBBaseUpdate	update	(elem) ;
	QString	 	id	= elem.attribute("id"    ) ;
	QString	 	expect	= elem.attribute("expect") ;
	QString		result	= update.getQueryText (m_dbLink) ;

	result	= result.simplifyWhiteSpace() ;
	expect	= expect.simplifyWhiteSpace() ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(id),
		_STDSTR(expect),
		_STDSTR(result)
	)	;
}

void	KBSQLTest::buildDeleteDOM
	(	const QDomElement	&elem
	)
{
	KBBaseDelete	delrec	(elem) ;
	QString	 	id	= elem.attribute("id"    ) ;
	QString	 	expect	= elem.attribute("expect") ;
	QString		result	= delrec.getQueryText (m_dbLink) ;

	result	= result.simplifyWhiteSpace() ;
	expect	= expect.simplifyWhiteSpace() ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(id),
		_STDSTR(expect),
		_STDSTR(result)
	)	;
}

void	KBSQLTest::buildSelectDOM
	(	const QDomElement	&elem
	)
{
	KBBaseSelect	select	(elem) ;
	QString		id	= elem.attribute("id"    ) ;
	QString		expect	= elem.attribute("expect") ;
	QString		result	= select.getQueryText (m_dbLink) ;

	result	= result.simplifyWhiteSpace() ;
	expect	= expect.simplifyWhiteSpace() ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(id),
		_STDSTR(expect),
		_STDSTR(result)
	)	;
}


void	KBSQLTest::evaluate
	(	const QDomElement	&elem
	)
{
	if (elem.nodeName() == "buildSelect")
	{
		buildSelect (elem) ;
		return	;
	}

	if (elem.nodeName() == "buildUpdate")
	{
		buildUpdate (elem) ;
		return	;
	}

	if (elem.nodeName() == "buildInsert")
	{
		buildInsert (elem) ;
		return	;
	}

	if (elem.nodeName() == "buildDelete")
	{
		buildDelete (elem) ;
		return	;
	}
}

void	KBSQLTest::evaluateDOM
	(	const QDomElement	&elem
	)
{
	if (elem.nodeName() == "buildSelect")
	{
		buildSelectDOM (elem) ;
		return	;
	}

	if (elem.nodeName() == "buildUpdate")
	{
		buildUpdateDOM (elem) ;
		return	;
	}

	if (elem.nodeName() == "buildInsert")
	{
		buildInsertDOM (elem) ;
		return	;
	}

	if (elem.nodeName() == "buildDelete")
	{
		buildDeleteDOM (elem) ;
		return	;
	}
}

void	KBSQLTest::testSQL ()
{
	evaluate (m_elem) ;
}

void	KBSQLTest::testSQLDOM ()
{
	evaluateDOM (m_elem) ;
}

static	void	addSQLTests
	(	CppUnit::TestSuite	*suite,
		const char		*spec
	)
{
	KBDomDocument	doc	;
	if (!doc.loadFile (spec, "test/common", getenv("TESTPATH")))
	{
		fprintf
		(	stderr,
			"sql: %s: cannot open: %s\n",
			spec,
			doc.lastError().getMessage().latin1()
		)	;
		exit	(1) ;
	}

	DOMITER_ELEM_BEGIN(doc.documentElement(),child)
	{
		suite->addTest
		(	new CppUnit::TestCaller<KBSQLTest>
			(	_STDSTR(child.attribute("id")),
				&KBSQLTest::testSQL,
				new KBSQLTest (_STDSTR(child.attribute("id")), child)
		)	)	;
	}
	DOMITER_END
}

static	void	addSQLTestsDOM
	(	CppUnit::TestSuite	*suite,
		const char		*spec
	)
{
	KBDomDocument	doc	;
	if (!doc.loadFile (spec, "test/common", getenv("TESTPATH")))
	{
		fprintf
		(	stderr,
			"sql: %s: cannot open: %s\n",
			spec,
			doc.lastError().getMessage().latin1()
		)	;
		exit	(1) ;
	}

	DOMITER_ELEM_BEGIN(doc.documentElement(),child)
	{
		suite->addTest
		(	new CppUnit::TestCaller<KBSQLTest>
			(	_STDSTR(child.attribute("id")),
				&KBSQLTest::testSQLDOM,
				new KBSQLTest (_STDSTR(child.attribute("id")), child)
		)	)	;
	}
	DOMITER_END
}

CppUnit::Test
	*KBSQLTest::suite ()
{
	CppUnit::TestSuite *s  = new CppUnit::TestSuite("SQL") ;

	if (rekallTestSkip("SQL")) return s ;

	CppUnit::TestSuite *sl = new CppUnit::TestSuite("SQL"   ) ;
	CppUnit::TestSuite *sd = new CppUnit::TestSuite("SQLDOM") ;

	s->addTest     (sl) ;
	s->addTest     (sd) ;

	addSQLTests    (sl, "baseSelect.xml") ;
	addSQLTests    (sl, "baseUpdate.xml") ;
	addSQLTests    (sl, "baseInsert.xml") ;
	addSQLTests    (sl, "baseDelete.xml") ;

	addSQLTestsDOM (sd, "baseSelect.xml") ;
	addSQLTestsDOM (sd, "baseUpdate.xml") ;
	addSQLTestsDOM (sd, "baseInsert.xml") ;
	addSQLTestsDOM (sd, "baseDelete.xml") ;

	return s ;
}

CppUnit::Test EXPORT_API *sqlTestSuite
	(	const QMap<QString,QString> *
	)
{
	return	KBSQLTest::suite () ;
}

