/***************************************************************************
    file	         : gridlayout.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qstring.h>
#include	<qmap.h>
#include	<qbutton.h>

#include	"rk_application.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_button.h"
#include	"kb_dom.h"
#include	"kb_ascii.h"
#include	"kb_formcopier.h"

#include	"tester.h"
#include	"tester_kbase.h"
#include	"accessor.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif



class	KBGridLayoutTest : public CppUnit::TestCase
{
	TestOpenForm		*m_form		;
	const char		*m_name		;
	const char		*m_text		;
	QRect			m_geom		;
	QRect			m_at		;

	KBButton		*getButton	(const char *, const char *) ;
	KBButton		*getButton	(KBNode    *) ;
	KBControl		*getControl	(KBButton  *) ;
	QButton			*getWidget	(KBControl *) ;

public	:

	KBGridLayoutTest	() ;
	KBGridLayoutTest
	(	std::string name,
		const char  *, const char  *,
		const QRect &, const QRect &
	)	;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testGridLayout	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBGridLayoutTest::KBGridLayoutTest ()
{
}

KBGridLayoutTest::KBGridLayoutTest
	(	std::string		tag,
		const char		*name,
		const char		*text,
		const QRect		&geom,
		const QRect		&at
	)
	:
	CppUnit::TestCase (tag),
	m_name		  (name),
	m_text		  (text),
	m_geom		  (geom),
	m_at		  (at)
{
}

void	KBGridLayoutTest::setUp ()
{
	KBError	error	;
	m_form	= rekallTestOpenFormMode (0, "design", "GridLayout", KB::ShowAsData, error) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage()),
		m_form != 0
	)	;
}

void	KBGridLayoutTest::tearDown ()
{
	rekallTestCloseForm () ;
}

void	KBGridLayoutTest::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBGridLayoutTest::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}

KBButton*KBGridLayoutTest::getButton
	(	const char	*name,
		const char	*text
	)
{
	KBNode	 *target = m_form->m_form->getNamedNode(name) ;

	CPPUNIT_ASSERT (target != 0) ;
	CPPUNIT_ASSERT (target->isA("KBButton")) ;

	KBButton *button = (KBButton *)target ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString(name)),
		_STDSTR(button->getName())
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString(text)),
		_STDSTR(button->getText())
	)	;

	return	button	;
}

KBButton*KBGridLayoutTest::getButton
	(	KBNode		*node
	)
{
	CPPUNIT_ASSERT (node != 0) ;
	CPPUNIT_ASSERT (node->isA("KBButton")) ;

	return	(KBButton *)node ;
}

KBControl
	*KBGridLayoutTest::getControl
	(	KBButton	*button
	)
{
	KBControl *control = KBTestAccessor::getControl (button) ;
	CPPUNIT_ASSERT (control != 0) ;
	CPPUNIT_ASSERT (control->isA("KBCtrlButton")) ;
	return	  control  ;
}

QButton	*KBGridLayoutTest::getWidget
	(	KBControl	*control
	)
{
	QWidget	*widget	  = control->getWidget() ;
	CPPUNIT_ASSERT (widget != 0) ;
	CPPUNIT_ASSERT (widget->isA("RKPushButton")) ;
	return	(QButton *)widget ;
}

void	KBGridLayoutTest::testGridLayout ()
{
	KBButton  *button  = getButton (m_name, m_text) ;
	KBControl *control = getControl(button ) ;
	QWidget	  *widget  = getWidget (control) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR
		(	QString	("Unexpected geometry: got %1, expected %2")
				.arg(_TEXT(control->ctrlGeometry ()))
				.arg(_TEXT(m_geom))
		),
		control->ctrlGeometry() == m_geom
	)	;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR
		(	QString	("Unexpected position: got %1, expected %2")
				.arg(_TEXT(button->realGeometry ()))
				.arg(_TEXT(m_at))
		),
		button->realGeometry() == m_at
	)	;
}

struct	GridLayoutTest
{
	const char	*m_tag		;
	const char	*m_name		;
	const char	*m_text		;
	QRect		m_geom		;
	QRect		m_at		;
}	;

static	GridLayoutTest	GridLayoutTests[]=
{
	{	"TopLeft",
		"topleft",
		"TopLeft",
		QRect(  0,  0,  1,  1),
		QRect(  0,  0,100, 30)
	},
	{	"TopMiddle",
		"topmiddle",
		"TopMiddle",
		QRect(  1,  0,  1,  1),
		QRect(175,  0,100, 30)
	},
	{	"TopRight",
		"topright",
		"TopRight",
		QRect(  2,  0,  1,  1),
		QRect(350,  0,100, 30)
	},
	{	"MiddleLeft",
		"middleleft",
		"MiddleLeft",
		QRect(  0,  1,  1,  1),
		QRect(  0, 60,100, 30)
	},
	{	"MiddleMiddle",
		"middlemiddle",
		"MiddleMiddle",
		QRect(  1,  1,  1,  1),
		QRect(175, 60,100, 30)
	},
	{	"MiddleRight",
		"middleright",
		"MiddleRight",
		QRect(  2,  1,  1,  1),
		QRect(350, 60,100, 30)
	},
	{	"BottomLeft",
		"bottomleft",
		"BottomLeft",
		QRect(  0,  2,  1,  1),
		QRect(  0,120,100, 30)
	},
	{	"BottomMiddle",
		"bottommiddle",
		"BottomMiddle",
		QRect(  1,  2,  1,  1),
		QRect(175,120,100, 30)
	},
	{	"BottomRight",
		"bottomright",
		"BottomRight",
		QRect(  2,  2,  1,  1),
		QRect(350,120,100, 30)
	}
}	;

CppUnit::Test
	*KBGridLayoutTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("GridLayout") ;

	if (rekallTestSkip ("GridLayout")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;


	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBGridLayoutTest> ("noDisplay", &KBGridLayoutTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBGridLayoutTest> ("withGUI",   &KBGridLayoutTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
		for (uint idx = 0 ; idx < sizeof(GridLayoutTests)/sizeof(GridLayoutTest) ; idx += 1)
		{
			GridLayoutTest *cpt = &GridLayoutTests[idx] ;

			s->addTest
			(	new CppUnit::TestCaller<KBGridLayoutTest>
				(	cpt->m_tag,
					&KBGridLayoutTest::testGridLayout,
					new KBGridLayoutTest
					(	cpt->m_tag,
						cpt->m_name,
						cpt->m_text,
						cpt->m_geom,
						cpt->m_at
					)
			)	)	;
		}

	return s ;
}

CppUnit::Test EXPORT_API *gridLayoutTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBGridLayoutTest::suite (__settings) ;
}
